@@ -45,6 +45,8 @@ namespace Gendarme {
 
 	public class XmlResultWriter : ResultWriter, IDisposable {
 
+		private const string AssemblySet = "[across all assemblies analyzed]";
+
 		XmlWriter writer;
 		XElement root;
 
@@ -53,7 +55,7 @@ namespace Gendarme {
 		{
 			writer = XmlWriter.Create (
 				CreateWriterFor (fileName),
-				new XmlWriterSettings { Indent = true, CloseOutput = true });
+				new XmlWriterSettings { Indent = true, CloseOutput = true, CheckCharacters = false });
 		}
 
 		static TextWriter CreateWriterFor (string fileName)
@@ -95,14 +97,14 @@ namespace Gendarme {
 				return "Type";
 			if (rule is IMethodRule)
 				return "Method";
-
-			throw new NotSupportedException ("RuleType not supported: " + rule.GetType ());
+			return "Other";
 		}
 
 		private XElement CreateRules ()
 		{
 			return new XElement ("rules",
 				from rule in Runner.Rules
+				where rule.Active
 				select CreateRule (rule, GetRuleType (rule)));
 		}
 
@@ -124,7 +126,7 @@ namespace Gendarme {
 					    Rule = a.Key,
 					    Value = from o in a
 						    group o by o.Target into r
-						    orderby r.Key.GetAssembly ().Name.FullName
+						    orderby (r.Key.GetAssembly () == null ? String.Empty : r.Key.GetAssembly ().Name.FullName)
 						    select new {
 							    Target = r.Key,
 							    Value = r
@@ -153,9 +155,10 @@ namespace Gendarme {
 
 		static XObject [] CreateTargetDetails (IMetadataTokenProvider target)
 		{
+			AssemblyDefinition assembly = target.GetAssembly ();
 			return new XObject [] {
 				new XAttribute ("Name", target.ToString ()),
-				new XAttribute ("Assembly", target.GetAssembly ().Name.FullName) };
+				new XAttribute ("Assembly", assembly == null ? AssemblySet : assembly.Name.FullName) };
 		}
 
 		static XElement CreateDefect (Defect defect)
