@@ -31,6 +31,7 @@
 using System;
 
 using Gendarme.Framework;
+using Gendarme.Framework.Engines;
 using Gendarme.Framework.Helpers;
 using Gendarme.Framework.Rocks;
 
@@ -39,8 +40,42 @@ using Mono.Cecil.Cil;
 
 namespace Gendarme.Rules.BadPractice {
 
+	/// <summary>
+	/// This rule checks for threads that are created but not started, returned or passed
+	/// to another method as an argument.
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// void UnusedThread ()
+	/// {
+	///	Thread thread = new Thread (threadStart);
+	///	thread.Name = "Thread 1";
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good examples:
+	/// <code>
+	/// void Start ()
+	/// {
+	///	Thread thread = new Thread (threadStart);
+	///	thread.Name = "Thread 1";
+	///	thread.Start ();
+	/// }
+	/// 
+	/// Thread InitializeThread ()
+	/// {
+	///	Thread thread = new Thread (threadStart);
+	///	thread.Name = "Thread 1";
+	///	return thread;
+	/// }
+	/// </code>
+	/// </example>
+
 	[Problem ("This method creates an thread that is never started nor returned to the caller.")]
 	[Solution ("Make sure the thread is required, start it (if it is) or remove it (if not).")]
+	[EngineDependency (typeof (OpCodeEngine))]
 	public class CheckNewThreadWithoutStartRule : Rule, IMethodRule {
 
 		private static bool CheckUsage (StackEntryUsageResult [] usageResults)
@@ -61,15 +96,15 @@ namespace Gendarme.Rules.BadPractice {
 				case Code.Stfld:
 					return true;
 				case Code.Call: //call (to the thread or as an argument)
-				case Code.Callvirt: {
-						MethodReference calledMethod = (MethodReference) usage.Instruction.Operand;
-						if (calledMethod.Parameters.Count <= usage.StackOffset) {
-							//thread.Method (not used as a parameter)
-							if (calledMethod.Name != "Start")
-								break;
-						}
-						return true;
+				case Code.Callvirt:
+					MethodReference calledMethod = (MethodReference) usage.Instruction.Operand;
+					int pcount = calledMethod.HasParameters ? calledMethod.Parameters.Count : 0;
+					if (pcount <= usage.StackOffset) {
+						//thread.Method (not used as a parameter)
+						if (calledMethod.Name != "Start")
+							break;
 					}
+					return true;
 				}
 			}
 			return false;
@@ -94,6 +129,10 @@ namespace Gendarme.Rules.BadPractice {
 			if (!method.HasBody)
 				return RuleResult.DoesNotApply;
 
+			// is there any Newobj instructions in this method
+			if (!OpCodeEngine.GetBitmask (method).Get (Code.Newobj))
+				return RuleResult.DoesNotApply;
+
 			StackEntryAnalysis sea = null;
 
 			foreach (Instruction ins in method.Body.Instructions) {
