@@ -35,16 +35,52 @@ using Mono.Cecil;
 using Mono.Cecil.Cil;
 
 using Gendarme.Framework;
+using Gendarme.Framework.Engines;
 using Gendarme.Framework.Helpers;
 using Gendarme.Framework.Rocks;
 
 namespace Gendarme.Rules.Interoperability {
 
+	/// <summary>
+	/// <code>Marshal.GetLastWin32Error()</code>should be called directly after a P/Invoke call. 
+	/// Intermediate method calls, even managed, could overwrite the error code.
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// public void DestroyError ()
+	/// {
+	///	MessageBeep (2);
+	///	Console.WriteLine ("Beep");
+	///	int error = Marshal.GetLastWin32Error ();
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example:
+	/// <code>
+	/// public void GetError ()
+	/// {
+	///	MessageBeep (2);
+	///	int error = Marshal.GetLastWin32Error ();
+	///	Console.WriteLine ("Beep");
+	/// }
+	/// 
+	/// public void DontUseGetLastError ()
+	/// {
+	///	MessageBeep (2);
+	///	Console.WriteLine ("Beep");
+	/// }
+	/// </code>
+	/// </example>
+
 	[Problem ("GetLastError() should be called immediately after this the P/Invoke call.")]
 	[Solution ("Move the call to GetLastError just after the P/Invoke call.")]
+	[EngineDependency (typeof (OpCodeEngine))]
+	[FxCopCompatibility ("Microsoft.Interoperability", "CA1404:CallGetLastErrorImmediatelyAfterPInvoke")]
 	public class GetLastErrorMustBeCalledRightAfterPInvokeRule : Rule, IMethodRule {
 
-		struct Branch {
+		struct Branch : IEquatable<Branch> {
 			public readonly Instruction Instruction;
 			public readonly bool DirtyMethodCalled;
 
@@ -82,14 +118,6 @@ namespace Gendarme.Rules.Interoperability {
 			}
 		}
 
-		private const string GetLastError = "System.Int32 System.Runtime.InteropServices.Marshal::GetLastWin32Error()";
-
-		static List<string> AllowedCalls = new List<string> () {
-			"System.Boolean System.Runtime.InteropServices.SafeHandle::get_IsInvalid()",
-			"System.Boolean System.IntPtr::op_Inequality(System.IntPtr,System.IntPtr)",
-			"System.Boolean System.IntPtr::op_Equality(System.IntPtr,System.IntPtr)"
-		};
-
 		List<Branch> branches = new List<Branch> ();
 
 		private bool CheckPInvoke (Instruction startInstruction)
@@ -107,21 +135,30 @@ namespace Gendarme.Rules.Interoperability {
 					//check if a method is called
 					if (ins.OpCode.FlowControl == FlowControl.Call) {
 
-						MethodDefinition mDef = ins.Operand as MethodDefinition;
+						MethodDefinition mDef = (ins.Operand as MethodReference).Resolve ();
 						if (mDef != null && mDef.IsPInvokeImpl) { //check if another pinvoke method is called, this counts as "GetLastError not called"
 							break;
 						}
-						string calledMethod = ins.Operand.ToString ();
 
-						if (calledMethod == GetLastError) {
-							getLastErrorFound = true;
+						string s = (mDef == null) ? String.Empty : mDef.DeclaringType.FullName;
+						switch (s) {
+						case "System.Runtime.InteropServices.Marshal":
+							getLastErrorFound = (mDef.Name == "GetLastWin32Error");
 							break; //found
-						}
-
-						if (!AllowedCalls.Contains (calledMethod)) {
+						case "System.Runtime.InteropServices.SafeHandle":
+							dirty = (mDef.Name != "get_IsInvalid");
+							break;
+						case "System.IntPtr":
+						case "System.UIntPtr":
+							dirty = ((mDef.Name != "op_Inequality") && (mDef.Name != "op_Equality"));
+							break;
+						default:
 							dirty = true;
+							break;
 						}
 
+						if (getLastErrorFound)
+							break;
 					}
 
 					//fetch the next instruction
@@ -161,6 +198,10 @@ namespace Gendarme.Rules.Interoperability {
 			if (!method.HasBody)
 				return RuleResult.DoesNotApply;
 
+			// avoid looping if we're sure there's no call in the method
+			if (!OpCodeBitmask.Calls.Intersect (OpCodeEngine.GetBitmask (method)))
+				return RuleResult.DoesNotApply;
+
 			foreach (Instruction ins in method.Body.Instructions) {
 				if (ins.OpCode.FlowControl != FlowControl.Call)
 					continue;
@@ -185,3 +226,4 @@ namespace Gendarme.Rules.Interoperability {
 		}
 	}
 }
+
