@@ -30,16 +30,84 @@ using System;
 
 using Mono.Cecil;
 using Mono.Cecil.Cil;
+
 using Gendarme.Framework;
+using Gendarme.Framework.Engines;
+using Gendarme.Framework.Helpers;
 using Gendarme.Framework.Rocks;
 using Gendarme.Rules.Performance;
 
 namespace Gendarme.Rules.Smells {
+	
+	/// <summary>
+	/// This rule allows developers to avoid the Speculative Generality smell. 
+	/// Be carefull if you are developing a new framework or a new library, 
+	/// because this rule only inspect the assembly, then if you provide an 
+	/// abstract base class for extend by thrid party people, then the rule
+	/// can warn you. You can ignore the message in this special case.
+	/// 
+	/// We can detect this kind of smell looking for some points:
+	/// <list type="bullet">
+    	/// <item><description>Abstract classes without responsability</description></item>
+    	/// <item><description>Unnecesary delegation.</description></item>
+	/// <item><description>Unused parameters.</description></item>
+    	/// </list>
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// // An abstract class with only one subclass.
+	/// public abstract class AbstractClass {
+	/// 	public abstract void MakeStuff ();
+	/// }
+	/// 
+	/// public class OverriderClass : AbstractClass {
+	/// 	public override void MakeStuff ()
+	///	{
+	///	}
+	/// }
+	/// </code>
+	/// If you use Telephone class only in one client, perhaps you don't need this kind of delegation. 
+	/// <code>
+	/// public class Person {
+	///	int age;
+	/// 	string name;
+	///	Telephone phone;
+	/// }
+ 	///
+	/// public class Telephone {
+	/// 	int areaCode;
+	/// 	int phone;
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example:
+	/// <code>
+	/// public abstract class OtherAbstractClass{
+	///	public abstract void MakeStuff ();
+	/// }
+	/// 
+	/// public class OtherOverriderClass : OtherAbstractClass {
+	/// 	public override void MakeStuff ()
+	/// 	{
+	/// 	}
+	/// }
+	/// 
+	/// public class YetAnotherOverriderClass : OtherAbstractClass {
+	/// 	public override void MakeStuff ()
+	/// 	{
+	/// 	}
+	/// }
+	/// </code>
+	/// </example>
+
 	[Problem ("If you will need the feature in the future then you should implement it in the future.")]
 	[Solution ("You can apply various refactorings: Collapse Hierarchy, Inline Class, Remove Parameter or Rename Method.")]
+	[EngineDependency (typeof (OpCodeEngine))]
 	public class AvoidSpeculativeGeneralityRule : Rule, ITypeRule {
 
-		public int CountInheritedClassesFrom (TypeDefinition baseType)
+		private int CountInheritedClassesFrom (TypeReference baseType)
 		{
 			int count = 0;
 			foreach (AssemblyDefinition assembly in Runner.Assemblies) {
@@ -68,10 +136,15 @@ namespace Gendarme.Rules.Smells {
 		{
 			if (!method.HasBody)
 				return false;
+
+			// avoid looping if we're sure there's no call in the method
+			if (!OpCodeBitmask.Calls.Intersect (OpCodeEngine.GetBitmask (method)))
+				return false;
+
 			bool onlyOneCallInstruction = false;
 
 			foreach (Instruction instruction in method.Body.Instructions) {
-				if (instruction.OpCode.Code == Code.Call || instruction.OpCode.Code == Code.Calli || instruction.OpCode.Code == Code.Callvirt)
+				if (instruction.OpCode.Code == Code.Call || instruction.OpCode.Code == Code.Callvirt)
 					if (onlyOneCallInstruction)
 						return false;
 					else
@@ -83,11 +156,14 @@ namespace Gendarme.Rules.Smells {
 
 		private static bool InheritsOnlyFromObject (TypeDefinition type)
 		{
-			return type.BaseType.FullName == "System.Object" && type.Interfaces.Count == 0;
+			return !type.HasInterfaces && type.BaseType.FullName == "System.Object";
 		}
 
 		private static bool MostlyMethodsDelegatesCall (TypeDefinition type)
 		{
+			if (!type.HasMethods)
+				return false; // 0 / 2 + 1 <= 0
+
 			int delegationCounter = 0;
 			foreach (MethodDefinition method in type.Methods) {
 				if (OnlyDelegatesCall (method))
@@ -135,7 +211,7 @@ namespace Gendarme.Rules.Smells {
 
 		public RuleResult CheckType (TypeDefinition type)
 		{
-			if (type.IsGeneratedCode ())
+			if (type.IsEnum || type.IsGeneratedCode ())
 				return RuleResult.DoesNotApply;
 
 			CheckAbstractClassWithoutResponsability (type);
