@@ -1,23 +1,39 @@
+//
+// CompareContext.cs
+//
+// (C) 2007 - 2008 Novell, Inc. (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+//
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
 
 using System;
-using System.Collections;
 using System.Collections.Generic;
 using System.Threading;
-using System.IO;
-using System.Xml;
-using Mono.Cecil;
-using Gtk;
 
 namespace GuiCompare {
-	
-	// A delegate used to load a CompAssembly
-	public delegate CompAssembly LoadCompAssembly ();
-	
+
 	public class CompareContext
 	{
-		LoadCompAssembly reference_loader, target_loader;
+		Func<CompAssembly> reference_loader, target_loader;
 		
-		public CompareContext (LoadCompAssembly reference, LoadCompAssembly target)
+		public CompareContext (Func<CompAssembly> reference, Func<CompAssembly> target)
 		{
 			reference_loader = reference;
 			target_loader = target;
@@ -40,29 +56,30 @@ namespace GuiCompare {
 		{
 		}
 
+		bool TryLoad (ref CompAssembly assembly, Func<CompAssembly> loader)
+		{
+			try {
+				assembly = loader ();
+				return true;
+			} catch (Exception e) {
+				OnError (e.ToString ());
+				return false;
+			}
+		}
+
 		void CompareThread ()
 		{
-			ProgressOnGuiThread (Double.NaN, "Loading reference...");
+			ProgressChange (Double.NaN, "Loading reference...");
 
-			try {
-				reference = reference_loader ();
-			}
-			catch (Exception e) {
-				ErrorOnGuiThread (e.ToString());
+			if (!TryLoad (ref reference, reference_loader))
 				return;
-			}
 
-			ProgressOnGuiThread (Double.NaN, "Loading target...");
+			ProgressChange (Double.NaN, "Loading target...");
 
-			try {
-				target = target_loader ();
-			}
-			catch (Exception e) {
-				ErrorOnGuiThread (e.ToString());
+			if (!TryLoad (ref target, target_loader))
 				return;
-			}
 
-			ProgressOnGuiThread (0.0, "Comparing...");
+			ProgressChange (0.0, "Comparing...");
 
 			comparison = target.GetComparisonNode ();
 
@@ -75,7 +92,7 @@ namespace GuiCompare {
 
 			CompareAttributes (comparison, reference, target);
 
-			FinishedOnGuiThread ();
+			Finish ();
 		}
 
 		int total_comparisons;
@@ -129,6 +146,15 @@ namespace GuiCompare {
 			                  reference_container.GetNestedDelegates(), target_container.GetNestedDelegates());
 		}
 
+		void CompareBaseTypes (ComparisonNode parent, ICompHasBaseType reference_type, ICompHasBaseType target_type)
+		{
+			if (reference_type.GetBaseType() != target_type.GetBaseType()) {
+				parent.AddError (String.Format ("reference has base class of {0}, target has base class of {1}",
+								reference_type.GetBaseType(),
+								target_type.GetBaseType()));
+			}
+		}
+
 		void CompareTypeLists (ComparisonNode parent,
 		                       List<CompNamed> reference_list,
 		                       List<CompNamed> target_list)
@@ -154,7 +180,7 @@ namespace GuiCompare {
 				comparisons_performed ++;
 				
 				if (c == 0) {
-					ProgressOnGuiThread ((double)comparisons_performed / total_comparisons * 100.0, String.Format ("Comparing {0} {1}", reference_list[m].Type, reference_list[m].Name));
+					ProgressChange ((double)comparisons_performed / total_comparisons * 100.0, String.Format ("Comparing {0} {1}", reference_list[m].Type, reference_list[m].Name));
 
 					/* the names match, further investigation is required */
 					ComparisonNode comparison = target_list[a].GetComparisonNode();
@@ -162,13 +188,9 @@ namespace GuiCompare {
 
 					// compare base types
 					if (reference_list[m] is ICompHasBaseType && target_list[a] is ICompHasBaseType) {
-						if (((ICompHasBaseType)reference_list[m]).GetBaseType() !=
-						    ((ICompHasBaseType)target_list[a]).GetBaseType()) {
-							comparison.AddError (String.Format ("reference type {0} has base class of {1}, target has base class of {2}",
-							                                    reference_list[m].Name,
-							                                    ((ICompHasBaseType)reference_list[m]).GetBaseType(),
-							                                    ((ICompHasBaseType)target_list[a]).GetBaseType()));
-						}
+						CompareBaseTypes (comparison,
+								  (ICompHasBaseType)reference_list[m],
+								  (ICompHasBaseType)target_list[a]);
 					}
 					
 					// compare nested types
@@ -182,6 +204,11 @@ namespace GuiCompare {
 						                (ICompMemberContainer)reference_list[m],
 						                (ICompMemberContainer)target_list[a]);
 					}
+					if (reference_list[m] is ICompAttributeContainer && target_list[a] is ICompAttributeContainer) {
+						CompareAttributes (comparison,
+								   (ICompAttributeContainer)reference_list[m],
+								   (ICompAttributeContainer)target_list[a]);
+					}
 
 					m++;
 					a++;
@@ -316,7 +343,7 @@ namespace GuiCompare {
 
 							comparison.AddError (String.Format ("reference access is '<i>{0}</i>', target access is '<i>{1}</i>'{2}",
 							                                    reference_access, target_access, extra_msg));
-							comparison.status = ComparisonStatus.Error;
+							comparison.Status = ComparisonStatus.Error;
 						}
 					}
 
@@ -324,7 +351,7 @@ namespace GuiCompare {
 						if (((CompMethod)target_list[a]).ThrowsNotImplementedException ()
 						    && !((CompMethod)reference_list[m]).ThrowsNotImplementedException ()) {
 							
-							comparison.throws_niex = true;
+							comparison.ThrowsNIE = true;
 						}
 					}
 
@@ -334,7 +361,7 @@ namespace GuiCompare {
 							comparison.AddError (String.Format ("reference field has value {0}, target field has value {1}",
 							                                    ((CompField)reference_list[m]).GetLiteralValue(),
 							                                    ((CompField)target_list[a]).GetLiteralValue()));
-							comparison.status = ComparisonStatus.Error;
+							comparison.Status = ComparisonStatus.Error;
 						}
 					}
 					
@@ -372,7 +399,7 @@ namespace GuiCompare {
 		{
 			ComparisonNode node = item.GetComparisonNode ();
 			parent.AddChild (node);
-			node.status = ComparisonStatus.Extra;
+			node.Status = ComparisonStatus.Extra;
 
 			if (item is ICompTypeContainer) {
 				ICompTypeContainer c = (ICompTypeContainer)item;
@@ -391,10 +418,21 @@ namespace GuiCompare {
 		{
 			ComparisonNode node = item.GetComparisonNode ();
 			parent.AddChild (node);
-			node.status = ComparisonStatus.Missing;
+			node.Status = ComparisonStatus.Missing;
 
 			comparisons_performed ++;
 
+			if (item is ICompHasBaseType) {
+				string baseTypeName = ((ICompHasBaseType)item).GetBaseType();
+				if (!string.IsNullOrEmpty (baseTypeName)) {
+					ComparisonNode baseTypeNode = new ComparisonNode (CompType.Class,
+											  string.Format ("BaseType: {0}",
+													 baseTypeName));
+					baseTypeNode.Status = ComparisonStatus.Missing;
+					node.AddChild (baseTypeNode);
+				}
+			}
+
 			if (item is ICompTypeContainer) {
 				ICompTypeContainer c = (ICompTypeContainer)item;
 
@@ -435,45 +473,32 @@ namespace GuiCompare {
 		// This is the new API.
 		CompAssembly target;
 
-		void ProgressOnGuiThread (double progress, string message)
+		void ProgressChange (double progress, string message)
 		{
-			Application.Invoke (delegate (object sender, EventArgs e) {
-				if (ProgressChanged != null)
-					ProgressChanged (this, new CompareProgressChangedEventArgs (message, progress));
-			});
+			if (ProgressChanged != null)
+				ProgressChanged (this, new CompareProgressChangedEventArgs (message, progress));
 		}
 
-		void ErrorOnGuiThread (string message)
+		void OnError (string message)
 		{
-			Application.Invoke (delegate (object sender, EventArgs e) {
-				if (Error != null)
-					Error (this, new CompareErrorEventArgs (message));
-			});
+			if (Error != null)
+				Error (this, new CompareErrorEventArgs (message));
 		}
 
-		void FinishedOnGuiThread ()
+		void Finish ()
 		{
-			Application.Invoke (delegate (object sender, EventArgs e) {
-				if (Finished != null)
-					Finished (this, EventArgs.Empty);
-			});
+			if (Finished != null)
+				Finished (this, EventArgs.Empty);
 		}
 
 		public event CompareProgressChangedEventHandler ProgressChanged;
 		public event CompareErrorEventHandler Error;
 		public event EventHandler Finished;
 
-		string masterinfoPath;
-		string assemblyPath;
 		ComparisonNode comparison;
 		Thread t;
 	}
 
-
-
-
-
-
 	public delegate void CompareProgressChangedEventHandler (object sender, CompareProgressChangedEventArgs args);
 	public delegate void CompareErrorEventHandler (object sender, CompareErrorEventArgs args);
 
