@@ -137,19 +137,8 @@ namespace GuiCompare
 			//"UIAutomationTypes",
 			//"WindowsFormsIntegration",	
 		};
-		
-		string [] api_sl11 = {
-			"mscorlib",
-			"System.Windows.dll",
-			"Microsoft.VisualBasic",
-			"System",
-			"System.Core",
-			"System.Net",
-			"System.Windows.Browser",
-			"System.Xml.Core",
-		};
 
-		string [] api_sl2_beta = {
+		string [] api_sl2 = {
 			"mscorlib",
 			"System.Windows",
 			"Microsoft.VisualBasic",
@@ -160,6 +149,11 @@ namespace GuiCompare
 			"System.ServiceModel",
 			"System.Windows.Browser",
 			"System.Xml",
+			"",
+			// sdk assemblies:
+			"System.Xml.Linq",
+			"System.Windows.Controls",
+			"System.Windows.Controls.Data",
 		};
 
 		string [] api_3_5 = {
@@ -211,32 +205,6 @@ namespace GuiCompare
 			"System.Messaging",
 		};
 		
-		string [] api_3_5_wxf = {			
-			"PresentationCore",
-			"PresentationFramework",
-			"System.Speech",
-			"WindowsBase",
-			"",
-			"System.IdentityModel",
-			"System.IdentityModel.Selectors",
-			"System.IO.Log",
-			"System.Runtime.Serialization",
-			"System.ServiceModel",
-			"",
-			"System.Workflow.Activities",
-			"System.Workflow.ComponentModel",
-			"System.Workflow.Runtime",
-			"",
-			"PresentationBuildTasks",
-			"",
-			"PresentationFramework.Aero",
-			"PresentationFramework.Classic",
-			"PresentationFramework.Luna",
-			"PresentationFramework.Royale",
-			"ReachFramework",
-			"System.Printing",
-		};
-		
 		public static void Init ()
 		{
 			
@@ -248,14 +216,12 @@ namespace GuiCompare
 			infos = Path.Combine (xdg_data_home, "mono-gui-compare");
 			Directory.CreateDirectory (infos);
 		}
-		
-		public delegate void MasterInfoLoaded (string path);
-		
+
 		// 
 		// Ensures that masterinfo file for assembly name @file in @prof profile
 		// is downloaded;   On success, executes the given callback
 		//
-		public void Ensure (string prof, string assemblyname, MasterInfoLoaded done)
+		public void Ensure (string prof, string assemblyname, Action<string> done)
 		{
 			string pdir = Path.Combine (infos, prof);
 			if (!Directory.Exists (pdir)){
@@ -268,30 +234,25 @@ namespace GuiCompare
 
 			switch (prof){
 			case "1.0":
-				u = new Uri ("http://mono.ximian.com/masterinfos/masterinfos-1.1.tar.gz");
+				u = new Uri ("http://mono.ximian.com/masterinfos/2.4/masterinfos-1.1.tar.gz");
 				break;
 				
 			case "2.0":
-				u = new Uri ("http://mono.ximian.com/masterinfos/masterinfos-2.0.tar.gz");
+				u = new Uri ("http://mono.ximian.com/masterinfos/2.4/masterinfos-2.0.tar.gz");
 				break;
 				
 			case "3.0":
-				u = new Uri ("http://mono.ximian.com/masterinfos/masterinfos-3.0.tar.gz");
-				break;
-				
-			case "2.1":
-				u = new Uri ("http://mono.ximian.com/masterinfos/masterinfos-sl11a-refresh.tar.gz");
-				break;
-
-			case "SL2":
-				//u = new Uri ("http://mono.ximian.com/masterinfos/masterinfos-sl2b1.tar.gz");
-				u = new Uri ("http://mono.ximian.com/masterinfos/masterinfos-sl2b2.tar.gz");
+				u = new Uri ("http://mono.ximian.com/masterinfos/2.4/masterinfos-3.0.tar.gz");
 				break;
 				
 			case "3.5":
-				u = new Uri ("http://mono.ximian.com/masterinfos/masterinfos-3.5.tar.gz");
+				u = new Uri ("http://mono.ximian.com/masterinfos/2.4/masterinfos-3.5.tar.gz");
 				break;
-				
+			
+			case "SL2":
+				u = new Uri ("http://mono.ximian.com/masterinfos/2.4/masterinfos-SL2.tar.gz");
+				break;
+			
 			default:
 				main.Status = "Profile is unknown";
 				return;
@@ -411,11 +372,16 @@ namespace GuiCompare
 				});
 			});
 		}
-		
+
 		void Populate (Menu container, string caption, string pdir, string collection, string [] elements)
 		{
 			string profiledir = System.IO.Path.Combine (monodir, pdir);
-			
+			string MONO_GAC_PREFIX = Environment.GetEnvironmentVariable ("MONO_GAC_PREFIX");
+			string[] gac_prefixes = null;
+
+			if (MONO_GAC_PREFIX != null)
+				gac_prefixes = MONO_GAC_PREFIX.Split (':');
+
 			MenuItem item = new MenuItem (caption);
 			Menu sub = new Menu ();
 			item.Submenu = sub;
@@ -429,12 +395,34 @@ namespace GuiCompare
 						continue;
 					child = new SeparatorMenuItem ();
 				} else {
-					string assemblyfile = System.IO.Path.Combine (profiledir, e + ".dll");
-					string element = e;
-					if (!System.IO.File.Exists (assemblyfile)){
-						Console.WriteLine ("Skipping {0} as {1} does not have it", e, profiledir);
+					string assemblyfile = null;
+					bool found = false;
+
+					if (gac_prefixes == null) {
+						assemblyfile = System.IO.Path.Combine (profiledir, e + ".dll");
+						found = System.IO.File.Exists (assemblyfile);
+					}
+					else {
+						foreach (string prefix in gac_prefixes) {
+							assemblyfile = System.IO.Path.Combine (
+									       System.IO.Path.Combine (
+										       System.IO.Path.Combine (
+											       System.IO.Path.Combine (prefix, "lib"),
+											       "mono"),
+										       pdir),
+									       e + ".dll");
+							found = System.IO.File.Exists (assemblyfile);
+							if (found)
+								break;
+						}
+					}
+
+					if (!found) {
+						Console.WriteLine ("Skipping {0} for profile {1}, could not locate it in profile dir or MONO_GAC_PREFIX", e, pdir);
 						continue;
 					}
+
+					string element = e;
 					child = new MenuItem (e);
 					child.Activated += delegate {
 						StartPresetCompare (assemblyfile, collection, element);
@@ -536,13 +524,10 @@ namespace GuiCompare
 			sub.Add (separator);
 			
 			Populate (sub, "API 1.1", "1.0", "1.0", api_1_1);
-			Populate (sub, "API 2.0", "2.0", "2.0", api_2_0);
-			Populate (sub, "API 3.0 (WxF)", "3.0", "3.0", api_3_0);
-			Populate (sub, "API 3.5 (2.0 SP1 + LINQ)", "2.0", "3.5", api_3_5);
-			Populate (sub, "API 3.5 (WxF SP1)", "3.0", "3.5", api_3_5_wxf);
-			
-			Populate (sub, "Silverlight 1.1 (Deprecated)", "2.1", "2.1", api_sl11);
-			Populate (sub, "Silverlight 2.0 (Beta 2)", "2.1", "SL2", api_sl2_beta);
+			Populate (sub, "API 2.0 sp1", "2.0", "2.0", api_2_0);
+			Populate (sub, "API 3.0 sp1", "3.0", "3.0", api_3_0);
+			Populate (sub, "API 3.5 sp1", "2.0", "3.5", api_3_5);
+			Populate (sub, "Silverlight 2.0", "2.1", "SL2", api_sl2);
 		}
 	}
 }
